"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
const utils_1 = require("./utils");
const util_1 = require("util");
function makeRouteAndOptions(initialOptions, method, controller, routeOrBody, bodyAs2ndParam) {
    utils_1.invariant(!(utils_1.isObject(routeOrBody) && utils_1.isObject(bodyAs2ndParam)), `If first argument of ${method.toLowerCase()}() is an object, you cannot have a 2nd argument. 😜`);
    utils_1.invariant(!(method === types_1.HTTPMethod.GET && utils_1.isObject(routeOrBody)), `You can only have query params as 1st argument of request.get()`);
    utils_1.invariant(!(method === types_1.HTTPMethod.GET && bodyAs2ndParam !== undefined), `You can only have query params as 1st argument of request.get()`);
    const route = (() => {
        if (routeOrBody instanceof URLSearchParams)
            return `?${routeOrBody}`;
        if (util_1.isString(routeOrBody))
            return routeOrBody;
        return '';
    })();
    const body = (() => {
        if (utils_1.isObject(routeOrBody))
            return JSON.stringify(routeOrBody);
        if (utils_1.isObject(bodyAs2ndParam))
            return JSON.stringify(bodyAs2ndParam);
        return JSON.stringify({});
    })();
    const options = (() => {
        const opts = Object.assign({}, initialOptions, { body,
            method, signal: controller.current ? controller.current.signal : null });
        if (routeOrBody instanceof URLSearchParams ||
            routeOrBody instanceof FormData ||
            bodyAs2ndParam instanceof FormData) {
            delete opts.headers['Content-Type'];
        }
        if (method === types_1.HTTPMethod.GET)
            delete opts.body;
        return opts;
    })();
    return {
        route,
        options,
    };
}
exports.default = makeRouteAndOptions;
//# sourceMappingURL=makeRouteAndOptions.js.map