export declare enum HTTPMethod {
    DELETE = "DELETE",
    GET = "GET",
    HEAD = "HEAD",
    OTIONS = "OPTIONS",
    PATCH = "PATCH",
    POST = "POST",
    PUT = "PUT"
}
export declare type FetchContextTypes = {
    url?: string;
    options?: RequestInit | undefined;
    graphql?: boolean;
};
export declare type BodyOnly = (body: BodyInit | object) => Promise<any>;
export declare type RouteOnly = (route: string) => Promise<any>;
export declare type RouteAndBodyOnly = (route: string, body: BodyInit | object) => Promise<any>;
export declare type NoArgs = () => Promise<any>;
export declare type FetchData = (routeOrBody?: string | BodyInit | object, body?: BodyInit | object) => Promise<any>;
export declare type RequestInitJSON = RequestInit & {
    headers: {
        'Content-Type': string;
    };
};
export interface FetchCommands {
    get: (route?: string) => Promise<any>;
    post: FetchData;
    patch: FetchData;
    put: FetchData;
    del: FetchData;
    delete: FetchData;
    query: (query: string, variables?: BodyInit | object) => Promise<any>;
    mutate: (mutation: string, variables?: BodyInit | object) => Promise<any>;
    abort: () => void;
}
export declare type DestructuringCommands<TData = any> = [TData | undefined, boolean, any, FetchCommands];
export interface UseFetchBaseResult<TData = any> {
    data: TData | undefined;
    loading: boolean;
    error: Error;
}
export declare type UseFetchResult<TData = any> = UseFetchBaseResult<TData> & FetchCommands & {
    request: FetchCommands;
};
export declare type UseFetch<TData> = DestructuringCommands<TData> & UseFetchResult<TData>;
export declare type CustomOptions = {
    onMount?: boolean;
    timeout?: number;
    url: string;
};
export declare type Options = CustomOptions & RequestInit;
export declare type NoUrlOptions = Omit<Options, 'url'>;
export declare type OptionsMaybeURL = NoUrlOptions & Partial<Pick<Options, 'url'>> & {
    url?: string;
};
export declare type OptionsOverwriteWithContext = (options: Options) => Options;
