"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
const react_1 = require("react");
const types_1 = require("./types");
const useCustomOptions_1 = __importDefault(require("./useCustomOptions"));
const useRequestInit_1 = __importDefault(require("./useRequestInit"));
const use_ssr_1 = __importDefault(require("use-ssr"));
const makeRouteAndOptions_1 = __importDefault(require("./makeRouteAndOptions"));
// TODO: handle context.graphql
function useFetch(urlOrOptions, optionsNoURLs) {
    const { isBrowser } = use_ssr_1.default();
    const { onMount, url } = useCustomOptions_1.default(urlOrOptions, optionsNoURLs);
    let requestInit = useRequestInit_1.default(urlOrOptions, optionsNoURLs);
    let controller = react_1.useRef();
    let data = react_1.useRef();
    // TODO: default config object should handle this
    const [loading, setLoading] = react_1.useState(onMount || false);
    const [error, setError] = react_1.useState();
    const makeFetch = react_1.useCallback((method) => {
        return (routeOrBody, body) => __awaiter(this, void 0, void 0, function* () {
            controller.current = isBrowser ? new AbortController() : null;
            const { route, options } = makeRouteAndOptions_1.default(requestInit, method, controller, routeOrBody, body);
            try {
                setLoading(true);
                const response = yield fetch(`${url}${route}`, options);
                try {
                    data.current = yield response.json();
                }
                catch (err) {
                    data.current = (yield response.text()); // FIXME: should not be `any` type
                }
            }
            catch (err) {
                if (err.name !== 'AbortError')
                    setError(err);
            }
            finally {
                controller.current = null;
                setLoading(false);
            }
            return data.current;
        });
    }, [url, isBrowser, requestInit]);
    const get = react_1.useCallback(makeFetch(types_1.HTTPMethod.GET), []);
    const post = react_1.useCallback(makeFetch(types_1.HTTPMethod.POST), []);
    const patch = react_1.useCallback(makeFetch(types_1.HTTPMethod.PATCH), []);
    const put = react_1.useCallback(makeFetch(types_1.HTTPMethod.PUT), []);
    const del = react_1.useCallback(makeFetch(types_1.HTTPMethod.DELETE), []);
    const query = react_1.useCallback((query, variables) => post({ query, variables }), [post]);
    const mutate = react_1.useCallback((mutation, variables) => post({ mutation, variables }), [post]);
    const abort = react_1.useCallback(() => { controller.current && controller.current.abort(); }, []);
    const request = react_1.useMemo(() => ({ get, post, patch, put, del, delete: del, abort, query, mutate }), [get, post, patch, put, del, abort, query, mutate]);
    // handling onMount
    react_1.useEffect(() => {
        if (!onMount)
            return;
        const methodName = requestInit.method || types_1.HTTPMethod.GET;
        if (!!url && methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodName.toLowerCase()];
            req(url, requestInit.body);
        }
        else if (!url && methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodName.toLowerCase()];
            req(requestInit.body);
        }
        else if (!!url) {
            const req = request[methodName.toLowerCase()];
            req(url);
        }
        else {
            const req = request[methodName.toLowerCase()];
            req();
        }
    }, [onMount, requestInit.body, requestInit.method, request, url]);
    return Object.assign([data.current, loading, error, request], Object.assign({ data: data.current, loading, error, request }, request));
}
exports.useFetch = useFetch;
exports.default = useFetch;
//# sourceMappingURL=useFetch.js.map